# -*- coding: utf-8 -*-
"""
Created : 2015-03-12

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate

tpl = DocxTemplate("templates/merge_paragraph_tpl.docx")

context = {
    "living_in_town": True,
}

tpl.render(context)
tpl.save("output/merge_paragraph.docx")



from docxtpl import DocxTemplate, R, Listing

tpl = DocxTemplate("templates/escape_tpl.docx")

context = {
    "myvar": R(
        '"less than" must be escaped : <, this can be done with RichText() or R()'
    ),
    "myescvar": 'It can be escaped with a "|e" jinja filter in the template too : < ',
    "nlnp": R(
        "Here is a multiple\nlines\nstring\aand some\aother\aparagraphs",
        color="#ff00ff",
    ),
    "mylisting": Listing("the listing\nwith\nsome\nlines\nand special chars : <>& ..."),
    "page_break": R("\f"),
    "new_listing": """
This is a new listing
Now, does not require Listing() Object
Here is a \t tab\a
Here is a new paragraph\a
Here is a page break : \f
That's it
""",
    "some_html": (
        "HTTP/1.1 200 OK\n"
        "Server: Apache-Coyote/1.1\n"
        "Cache-Control: no-store\n"
        "Expires: Thu, 01 Jan 1970 00:00:00 GMT\n"
        "Pragma: no-cache\n"
        "Content-Type: text/html;charset=UTF-8\n"
        "Content-Language: zh-CN\n"
        "Date: Thu, 22 Oct 2020 10:59:40 GMT\n"
        "Content-Length: 9866\n"
        "\n"
        "<html>\n"
        "<head>\n"
        "    <title>Struts Problem Report</title>\n"
        "    <style>\n"
        "    \tpre {\n"
        "\t    \tmargin: 0;\n"
        "\t        padding: 0;\n"
        "\t    }    "
        "\n"
        "    </style>\n"
        "</head>\n"
        "<body>\n"
        "...\n"
        "</body>\n"
        "</html>"
    ),
}

tpl.render(context)
tpl.save("output/escape.docx")



# -*- coding: utf-8 -*-
"""
Created : 2024-09-23

@author: Bart Broere
"""

from docxtpl import DocxTemplate

DEST_FILE = "output/footnotes.docx"

tpl = DocxTemplate("templates/footnotes_tpl.docx")

context = {"a_jinja_variable": "A Jinja variable!"}

tpl.render(context)
tpl.save(DEST_FILE)



"""
@author: Max Podolskii
"""

import os
from unicodedata import name

from docxtpl import DocxTemplate


XML_RESERVED = """<"&'>"""

tpl = DocxTemplate("templates/escape_tpl_auto.docx")

context = {
    "nested_dict": {name(str(c)): c for c in XML_RESERVED},
    "autoescape": 'Escaped "str & ing"!',
    "autoescape_unicode": "This is an escaped <unicode> example \u4f60 & \u6211",
    "iteritems": lambda x: x.items(),
}

tpl.render(context, autoescape=True)

OUTPUT = "output"
if not os.path.exists(OUTPUT):
    os.makedirs(OUTPUT)
tpl.save(OUTPUT + "/escape_auto.docx")



from docxtpl import DocxTemplate

# With old docxtpl version, "... for spicy ..." was replaced by "... forspicy..."
# This test is for checking that is some cases the spaces are not lost anymore

tpl = DocxTemplate("templates/preserve_spaces_tpl.docx")

tags = ["tag_1", "tag_2"]
replacement = ["looking", "too"]

context = dict(zip(tags, replacement))

tpl.render(context)
tpl.save("output/preserve_spaces.docx")



# -*- coding: utf-8 -*-
"""
Created : 2015-03-12

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate

tpl = DocxTemplate("templates/header_footer_tpl.docx")

sd = tpl.new_subdoc()
p = sd.add_paragraph(
    "This is a sub-document to check it does not break header and footer"
)

context = {
    "title": "Header and footer test",
    "company_name": "The World Wide company",
    "date": "2016-03-17",
    "mysubdoc": sd,
}

tpl.render(context)
tpl.save("output/header_footer.docx")



# -*- coding: utf-8 -*-
"""
Created : 2021-07-30

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate


tpl = DocxTemplate("templates/merge_docx_master_tpl.docx")
sd = tpl.new_subdoc("templates/merge_docx_subdoc.docx")

context = {
    "mysubdoc": sd,
}

tpl.render(context)
tpl.save("output/merge_docx.docx")



# -*- coding: utf-8 -*-
"""
Created : 2015-03-12

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate
from docx.shared import Inches

tpl = DocxTemplate("templates/subdoc_tpl.docx")

sd = tpl.new_subdoc()
p = sd.add_paragraph("This is a sub-document inserted into a bigger one")
p = sd.add_paragraph("It has been ")
p.add_run("dynamically").style = "dynamic"
p.add_run(" generated with python by using ")
p.add_run("python-docx").italic = True
p.add_run(" library")

sd.add_heading("Heading, level 1", level=1)
sd.add_paragraph("This is an Intense quote", style="IntenseQuote")

sd.add_paragraph("A picture :")
sd.add_picture("templates/python_logo.png", width=Inches(1.25))

sd.add_paragraph("A Table :")
table = sd.add_table(rows=1, cols=3)
hdr_cells = table.rows[0].cells
hdr_cells[0].text = "Qty"
hdr_cells[1].text = "Id"
hdr_cells[2].text = "Desc"
recordset = ((1, 101, "Spam"), (2, 42, "Eggs"), (3, 631, "Spam,spam, eggs, and ham"))
for item in recordset:
    row_cells = table.add_row().cells
    row_cells[0].text = str(item[0])
    row_cells[1].text = str(item[1])
    row_cells[2].text = item[2]

context = {
    "mysubdoc": sd,
}

tpl.render(context)
tpl.save("output/subdoc.docx")



# -*- coding: utf-8 -*-
"""
Created : 2015-03-26

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate, RichText

tpl = DocxTemplate("templates/richtext_tpl.docx")

rt = RichText()
rt.add("a rich text", style="myrichtextstyle")
rt.add(" with ")
rt.add("some italic", italic=True)
rt.add(" and ")
rt.add("some violet", color="#ff00ff")
rt.add(" and ")
rt.add("some striked", strike=True)
rt.add(" and ")
rt.add("some Highlighted", highlight="#ffff00")
rt.add(" and ")
rt.add("some small", size=14)
rt.add(" or ")
rt.add("big", size=60)
rt.add(" text.")
rt.add("\nYou can add an hyperlink, here to ")
rt.add("google", url_id=tpl.build_url_id("http://google.com"))
rt.add("\nEt voilà ! ")
rt.add("\n1st line")
rt.add("\n2nd line")
rt.add("\n3rd line")
rt.add("\aA new paragraph : <cool>\a")
rt.add("--- A page break here (see next page) ---\f")

for ul in [
    "single",
    "double",
    "thick",
    "dotted",
    "dash",
    "dotDash",
    "dotDotDash",
    "wave",
]:
    rt.add("\nUnderline : " + ul + " \n", underline=ul)
rt.add("\nFonts :\n", underline=True)
rt.add("Arial\n", font="Arial")
rt.add("Courier New\n", font="Courier New")
rt.add("Times New Roman\n", font="Times New Roman")
rt.add("\n\nHere some")
rt.add("superscript", superscript=True)
rt.add(" and some")
rt.add("subscript", subscript=True)

rt_embedded = RichText("an example of ")
rt_embedded.add(rt)

context = {
    "example": rt_embedded,
}

tpl.render(context)
tpl.save("output/richtext.docx")



import subprocess
import glob
import os

tests = sorted(glob.glob("[A-Za-z]*.py"))
excludes = ["runtests.py"]

output_dir = os.path.join(os.path.dirname(__file__), "output")
if not os.path.exists(output_dir):
    os.mkdir(output_dir)

for test in tests:
    if test not in excludes:
        print("%s ..." % test)
        subprocess.call(["python", "./%s" % test])

print("Done.")



# -*- coding: utf-8 -*-
"""
Created : 2017-10-15

@author: Arthaslixin
"""

from docxtpl import DocxTemplate

tpl = DocxTemplate("templates/vertical_merge_tpl.docx")

context = {
    "items": [
        {"desc": "Python interpreters", "qty": 2, "price": "FREE"},
        {"desc": "Django projects", "qty": 5403, "price": "FREE"},
        {"desc": "Guido", "qty": 1, "price": "100,000,000.00"},
    ],
    "total_price": "100,000,000.00",
    "category": "Book",
}

tpl.render(context)
tpl.save("output/vertical_merge.docx")



# -*- coding: utf-8 -*-
"""
Created : 2021-07-30

@author: Eric Lapouyade
"""
try:
    from html import escape
except ImportError:
    # cgi.escape is deprecated in python 3.7
    from cgi import escape


class RichText(object):
    """class to generate Rich Text when using templates variables

    This is much faster than using Subdoc class,
    but this only for texts INSIDE an existing paragraph.
    """

    def __init__(self, text=None, **text_prop):
        self.xml = ""
        if text:
            self.add(text, **text_prop)

    def add(
        self,
        text,
        style=None,
        color=None,
        highlight=None,
        size=None,
        subscript=None,
        superscript=None,
        bold=False,
        italic=False,
        underline=False,
        strike=False,
        font=None,
        url_id=None,
        rtl=False,
        lang=None,
    ):

        # If a RichText is added
        if isinstance(text, RichText):
            self.xml += text.xml
            return

        # # If nothing to add : just return
        # if text is None or text == "":
        #     return

        # If not a string : cast to string (ex: int, dict etc...)
        if not isinstance(text, (str, bytes)):
            text = str(text)
        if not isinstance(text, str):
            text = text.decode("utf-8", errors="ignore")
        text = escape(text)

        prop = ""

        if style:
            prop += '<w:rStyle w:val="%s"/>' % style
        if color:
            if color[0] == "#":
                color = color[1:]
            prop += '<w:color w:val="%s"/>' % color
        if highlight:
            if highlight[0] == "#":
                highlight = highlight[1:]
            prop += '<w:shd w:fill="%s"/>' % highlight
        if size:
            prop += '<w:sz w:val="%s"/>' % size
            prop += '<w:szCs w:val="%s"/>' % size
        if subscript:
            prop += '<w:vertAlign w:val="subscript"/>'
        if superscript:
            prop += '<w:vertAlign w:val="superscript"/>'
        if bold:
            prop += "<w:b/>"
            if rtl:
                prop += "<w:bCs/>"
        if italic:
            prop += "<w:i/>"
            if rtl:
                prop += "<w:iCs/>"
        if underline:
            if underline not in [
                "single",
                "double",
                "thick",
                "dotted",
                "dash",
                "dotDash",
                "dotDotDash",
                "wave",
            ]:
                underline = "single"
            prop += '<w:u w:val="%s"/>' % underline
        if strike:
            prop += "<w:strike/>"
        if font:
            regional_font = ""
            if ":" in font:
                region, font = font.split(":", 1)
                regional_font = ' w:{region}="{font}"'.format(font=font, region=region)
            prop += '<w:rFonts w:ascii="{font}" w:hAnsi="{font}" w:cs="{font}"{regional_font}/>'.format(
                font=font, regional_font=regional_font
            )
        if rtl:
            prop += '<w:rtl w:val="true"/>'
        if lang:
            prop += '<w:lang w:val="%s"/>' % lang
        xml = "<w:r>"
        if prop:
            xml += "<w:rPr>%s</w:rPr>" % prop
        xml += '<w:t xml:space="preserve">%s</w:t></w:r>' % text
        if url_id:
            xml = '<w:hyperlink r:id="%s" w:tgtFrame="_blank">%s</w:hyperlink>' % (
                url_id,
                xml,
            )
        self.xml += xml

    def __unicode__(self):
        return self.xml

    def __str__(self):
        return self.xml

    def __html__(self):
        return self.xml


class RichTextParagraph(object):
    """class to generate Rich Text Paragraphs when using templates variables

    This is much faster than using Subdoc class,
    but this only for texts OUTSIDE an existing paragraph.
    """

    def __init__(self, text=None, **text_prop):
        self.xml = ""
        if text:
            self.add(text, **text_prop)

    def add(
        self,
        text,
        parastyle=None,
    ):

        # If a RichText is added
        if not isinstance(text, RichText):
            text = RichText(text)

        prop = ""
        if parastyle:
            prop += '<w:pStyle w:val="%s"/>' % parastyle

        xml = "<w:p>"
        if prop:
            xml += "<w:pPr>%s</w:pPr>" % prop
        xml += text.xml
        xml += "</w:p>"
        self.xml += xml

    def __unicode__(self):
        return self.xml

    def __str__(self):
        return self.xml

    def __html__(self):
        return self.xml


R = RichText
RP = RichTextParagraph



from docxtpl import DocxTemplate
from jinja2.exceptions import TemplateError

print("=" * 80)
print("Generating template error for testing (so it is safe to ignore) :")
print("." * 80)
try:
    tpl = DocxTemplate("templates/template_error_tpl.docx")
    tpl.render({"test_variable": "test variable value"})
except TemplateError as the_error:
    print(str(the_error))
    if hasattr(the_error, "docx_context"):
        print("Context:")
        for line in the_error.docx_context:
            print(line)
tpl.save("output/template_error.docx")
print("." * 80)
print(" End of TemplateError Test ")
print("=" * 80)



from docxtpl import DocxTemplate

doctemplate = r"templates/doc_properties_tpl.docx"

tpl = DocxTemplate(doctemplate)

context = {"test": "HelloWorld"}

tpl.render(context)
tpl.save("output/doc_properties.docx")



# -*- coding: utf-8 -*-
"""
Created : 2015-03-26

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate, RichText

tpl = DocxTemplate("templates/richtext_and_if_tpl.docx")


context = {"foobar": RichText("Foobar!", color="ff0000")}

tpl.render(context)
tpl.save("output/richtext_and_if.docx")



from docxtpl import DocxTemplate, RichText

tpl = DocxTemplate("templates/word2016_tpl.docx")
tpl.render(
    {
        "test_space": "          ",
        "test_tabs": 5 * "\t",
        "test_space_r": RichText("          "),
        "test_tabs_r": RichText(5 * "\t"),
    }
)
tpl.save("output/word2016.docx")



# -*- coding: utf-8 -*-
"""
Created : 2015-03-12

@author: Eric Lapouyade
"""
__version__ = "0.19.1"

# flake8: noqa
from .inline_image import InlineImage
from .listing import Listing
from .richtext import RichText, R, RichTextParagraph, RP
from .subdoc import Subdoc
from .template import DocxTemplate



# -*- coding: utf-8 -*-
"""
Created : 2017-09-03

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate

DEST_FILE = "output/header_footer_image.docx"

tpl = DocxTemplate("templates/header_footer_image_tpl.docx")

context = {
    "mycompany": "The World Wide company",
}
tpl.replace_media("templates/dummy_pic_for_header.png", "templates/python.png")
tpl.render(context)
tpl.save(DEST_FILE)



# -*- coding: utf-8 -*-
"""
Created : 2019-05-22

@author: Eric Dufresne
"""

from docxtpl import DocxTemplate
import io

DEST_FILE = "output/header_footer_image_file_obj.docx"
DEST_FILE2 = "output/header_footer_image_file_obj2.docx"

tpl = DocxTemplate("templates/header_footer_image_tpl.docx")

context = {
    "mycompany": "The World Wide company",
}

dummy_pic = io.BytesIO(open("templates/dummy_pic_for_header.png", "rb").read())
new_image = io.BytesIO(open("templates/python.png", "rb").read())
tpl.replace_media(dummy_pic, new_image)
tpl.render(context)
tpl.save(DEST_FILE)

tpl = DocxTemplate("templates/header_footer_image_tpl.docx")
dummy_pic.seek(0)
new_image.seek(0)
tpl.replace_media(dummy_pic, new_image)
tpl.render(context)

file_obj = io.BytesIO()
tpl.save(file_obj)
file_obj.seek(0)
with open(DEST_FILE2, "wb") as f:
    f.write(file_obj.read())



# -*- coding: utf-8 -*-
"""
Created : 2017-09-03

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate

DEST_FILE = "output/replace_picture.docx"

tpl = DocxTemplate("templates/replace_picture_tpl.docx")

context = {}

tpl.replace_pic("python_logo.png", "templates/python.png")
tpl.render(context)
tpl.save(DEST_FILE)



# -*- coding: utf-8 -*-
"""
Created : 2017-01-14

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate, InlineImage

# for height and width you have to use millimeters (Mm), inches or points(Pt) class :
from docx.shared import Mm
import jinja2

tpl = DocxTemplate("templates/inline_image_tpl.docx")

context = {
    "myimage": InlineImage(tpl, "templates/python_logo.png", width=Mm(20)),
    "myimageratio": InlineImage(
        tpl, "templates/python_jpeg.jpg", width=Mm(30), height=Mm(60)
    ),
    "frameworks": [
        {
            "image": InlineImage(tpl, "templates/django.png", height=Mm(10)),
            "desc": "The web framework for perfectionists with deadlines",
        },
        {
            "image": InlineImage(tpl, "templates/zope.png", height=Mm(10)),
            "desc": "Zope is a leading Open Source Application Server "
            "and Content Management Framework",
        },
        {
            "image": InlineImage(tpl, "templates/pyramid.png", height=Mm(10)),
            "desc": "Pyramid is a lightweight Python web framework aimed at taking "
            "small web apps into big web apps.",
        },
        {
            "image": InlineImage(tpl, "templates/bottle.png", height=Mm(10)),
            "desc": "Bottle is a fast, simple and lightweight WSGI micro web-framework "
            "for Python",
        },
        {
            "image": InlineImage(tpl, "templates/tornado.png", height=Mm(10)),
            "desc": "Tornado is a Python web framework and asynchronous networking "
            "library.",
        },
    ],
}
# testing that it works also when autoescape has been forced to True
jinja_env = jinja2.Environment(autoescape=True)
tpl.render(context, jinja_env)
tpl.save("output/inline_image.docx")



# -*- coding: utf-8 -*-
"""
Created : 2015-03-12

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate

tpl = DocxTemplate("templates/order_tpl.docx")

context = {
    "customer_name": "Eric",
    "items": [
        {"desc": "Python interpreters", "qty": 2, "price": "FREE"},
        {"desc": "Django projects", "qty": 5403, "price": "FREE"},
        {"desc": "Guido", "qty": 1, "price": "100,000,000.00"},
    ],
    "in_europe": True,
    "is_paid": False,
    "company_name": "The World Wide company",
    "total_price": "100,000,000.00",
}

tpl.render(context)
tpl.save("output/order.docx")



import argparse
import json
import os

from .template import DocxTemplate, TemplateError

TEMPLATE_ARG = "template_path"
JSON_ARG = "json_path"
OUTPUT_ARG = "output_filename"
OVERWRITE_ARG = "overwrite"
QUIET_ARG = "quiet"


def make_arg_parser():
    parser = argparse.ArgumentParser(
        usage="python -m docxtpl [-h] [-o] [-q] {} {} {}".format(
            TEMPLATE_ARG, JSON_ARG, OUTPUT_ARG
        ),
        description="Make docx file from existing template docx and json data.",
    )
    parser.add_argument(
        TEMPLATE_ARG, type=str, help="The path to the template docx file."
    )
    parser.add_argument(
        JSON_ARG, type=str, help="The path to the json file with the data."
    )
    parser.add_argument(
        OUTPUT_ARG, type=str, help="The filename to save the generated docx."
    )
    parser.add_argument(
        "-" + OVERWRITE_ARG[0],
        "--" + OVERWRITE_ARG,
        action="store_true",
        help="If output file already exists, overwrites without asking for confirmation",
    )
    parser.add_argument(
        "-" + QUIET_ARG[0],
        "--" + QUIET_ARG,
        action="store_true",
        help="Do not display unnecessary messages",
    )
    return parser


def get_args(parser):
    try:
        parsed_args = vars(parser.parse_args())
        return parsed_args
    # Argument errors raise a SystemExit with code 2. Normal usage of the
    # --help or -h flag raises a SystemExit with code 0.
    except SystemExit as e:
        if e.code == 0:
            raise SystemExit
        else:
            raise RuntimeError(
                "Correct usage is:\n{parser.usage}".format(parser=parser)
            )


def is_argument_valid(arg_name, arg_value, overwrite):
    # Basic checks for the arguments
    if arg_name == TEMPLATE_ARG:
        return os.path.isfile(arg_value) and arg_value.endswith(".docx")
    elif arg_name == JSON_ARG:
        return os.path.isfile(arg_value) and arg_value.endswith(".json")
    elif arg_name == OUTPUT_ARG:
        return arg_value.endswith(".docx") and check_exists_ask_overwrite(
            arg_value, overwrite
        )
    elif arg_name in [OVERWRITE_ARG, QUIET_ARG]:
        return arg_value in [True, False]


def check_exists_ask_overwrite(arg_value, overwrite):
    # If output file does not exist or command was run with overwrite option,
    # returns True, else asks for overwrite confirmation. If overwrite is
    # confirmed returns True, else raises OSError.
    if os.path.exists(arg_value) and not overwrite:
        try:
            msg = (
                "File %s already exists, would you like to overwrite the existing file? "
                "(y/n)" % arg_value
            )
            if input(msg).lower() == "y":
                return True
            else:
                raise OSError
        except OSError:
            raise RuntimeError(
                "File %s already exists, please choose a different name." % arg_value
            )
    else:
        return True


def validate_all_args(parsed_args):
    overwrite = parsed_args[OVERWRITE_ARG]
    # Raises AssertionError if any of the arguments is not validated
    try:
        for arg_name, arg_value in parsed_args.items():
            if not is_argument_valid(arg_name, arg_value, overwrite):
                raise AssertionError
    except AssertionError:
        raise RuntimeError(
            'The specified {arg_name} "{arg_value}" is not valid.'.format(
                arg_name=arg_name, arg_value=arg_value
            )
        )


def get_json_data(json_path):
    with open(json_path) as file:
        try:
            json_data = json.load(file)
            return json_data
        except json.JSONDecodeError as e:
            print(
                "There was an error on line {e.lineno}, column {e.colno} while trying "
                "to parse file {json_path}".format(e=e, json_path=json_path)
            )
            raise RuntimeError("Failed to get json data.")


def make_docxtemplate(template_path):
    try:
        return DocxTemplate(template_path)
    except TemplateError:
        raise RuntimeError("Could not create docx template.")


def render_docx(doc, json_data):
    try:
        doc.render(json_data)
        return doc
    except TemplateError:
        raise RuntimeError("An error ocurred while trying to render the docx")


def save_file(doc, parsed_args):
    try:
        output_path = parsed_args[OUTPUT_ARG]
        doc.save(output_path)
        if not parsed_args[QUIET_ARG]:
            print(
                "Document successfully generated and saved at {output_path}".format(
                    output_path=output_path
                )
            )
    except OSError as e:
        print("{e.strerror}. Could not save file {e.filename}.".format(e=e))
        raise RuntimeError("Failed to save file.")


def main():
    parser = make_arg_parser()
    # Everything is in a try-except block that catches a RuntimeError that is
    # raised if any of the individual functions called cause an error
    # themselves, terminating the main function.
    parsed_args = get_args(parser)
    try:
        validate_all_args(parsed_args)
        json_data = get_json_data(os.path.abspath(parsed_args[JSON_ARG]))
        doc = make_docxtemplate(os.path.abspath(parsed_args[TEMPLATE_ARG]))
        doc = render_docx(doc, json_data)
        save_file(doc, parsed_args)
    except RuntimeError as e:
        print("Error: " + e.__str__())
        return
    finally:
        if not parsed_args[QUIET_ARG]:
            print("Exiting program!")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
"""
Created : 2021-04-06

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate, InlineImage

# for height and width you have to use millimeters (Mm), inches or points(Pt) class :
from docx.shared import Mm

tpl = DocxTemplate("templates/header_footer_inline_image_tpl.docx")

context = {
    "inline_image": InlineImage(tpl, "templates/django.png", height=Mm(10)),
    "images": [
        InlineImage(tpl, "templates/python.png", height=Mm(10)),
        InlineImage(tpl, "templates/python.png", height=Mm(10)),
        InlineImage(tpl, "templates/python.png", height=Mm(10)),
    ],
}
tpl.render(context)
tpl.save("output/header_footer_inline_image.docx")



# -*- coding: utf-8 -*-
"""
Created : 2021-07-30

@author: Eric Lapouyade
"""
from docx.oxml import OxmlElement, parse_xml
from docx.oxml.ns import qn


class InlineImage(object):
    """Class to generate an inline image

    This is much faster than using Subdoc class.
    """

    tpl = None
    image_descriptor = None
    width = None
    height = None
    anchor = None

    def __init__(self, tpl, image_descriptor, width=None, height=None, anchor=None):
        self.tpl, self.image_descriptor = tpl, image_descriptor
        self.width, self.height = width, height
        self.anchor = anchor

    def _add_hyperlink(self, run, url, part):
        # Create a relationship for the hyperlink
        r_id = part.relate_to(
            url,
            "http://schemas.openxmlformats.org/officeDocument/2006/relationships/hyperlink",
            is_external=True,
        )

        # Find the <wp:docPr> and <pic:cNvPr> element
        docPr = run.xpath(".//wp:docPr")[0]
        cNvPr = run.xpath(".//pic:cNvPr")[0]

        # Create the <a:hlinkClick> element
        hlinkClick1 = OxmlElement("a:hlinkClick")
        hlinkClick1.set(qn("r:id"), r_id)
        hlinkClick2 = OxmlElement("a:hlinkClick")
        hlinkClick2.set(qn("r:id"), r_id)

        # Insert the <a:hlinkClick> element right after the <wp:docPr> element
        docPr.append(hlinkClick1)
        cNvPr.append(hlinkClick2)

        return run

    def _insert_image(self):
        pic = self.tpl.current_rendering_part.new_pic_inline(
            self.image_descriptor,
            self.width,
            self.height,
        ).xml
        if self.anchor:
            run = parse_xml(pic)
            if run.xpath(".//a:blip"):
                hyperlink = self._add_hyperlink(
                    run, self.anchor, self.tpl.current_rendering_part
                )
                pic = hyperlink.xml

        return (
            "</w:t></w:r><w:r><w:drawing>%s</w:drawing></w:r><w:r>"
            '<w:t xml:space="preserve">' % pic
        )

    def __unicode__(self):
        return self._insert_image()

    def __str__(self):
        return self._insert_image()

    def __html__(self):
        return self._insert_image()



# -*- coding: utf-8 -*-
"""
Created : 2015-03-12

@author: sandeeprah, Eric Lapouyade
"""

from docxtpl import DocxTemplate
import jinja2

jinja_env = jinja2.Environment()


# to create new filters, first create functions that accept the value to filter
# as first argument, and filter parameters as next arguments
def my_filterA(value, my_string_arg):
    return_value = value + " " + my_string_arg
    return return_value


def my_filterB(value, my_float_arg):
    return_value = value + my_float_arg
    return return_value


# Then, declare them to jinja like this :
jinja_env.filters["my_filterA"] = my_filterA
jinja_env.filters["my_filterB"] = my_filterB


context = {"base_value_string": " Hello", "base_value_float": 1.5}

tpl = DocxTemplate("templates/custom_jinja_filters_tpl.docx")
tpl.render(context, jinja_env)
tpl.save("output/custom_jinja_filters.docx")



# -*- coding: utf-8 -*-
"""
Created : 2017-09-09

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate

# rendering the "dynamic embedded docx":
embedded_docx_tpl = DocxTemplate("templates/embedded_embedded_docx_tpl.docx")
context = {
    "name": "John Doe",
}
embedded_docx_tpl.render(context)
embedded_docx_tpl.save("output/embedded_embedded_docx.docx")


# rendering the main document :
tpl = DocxTemplate("templates/embedded_main_tpl.docx")

context = {
    "name": "John Doe",
}

tpl.replace_embedded(
    "templates/embedded_dummy.docx", "templates/embedded_static_docx.docx"
)
tpl.replace_embedded(
    "templates/embedded_dummy2.docx", "output/embedded_embedded_docx.docx"
)

# The zipname is the one you can find when you open docx with WinZip, 7zip (Windows)
# or unzip -l (Linux). The zipname starts with "word/embeddings/".
# Note that the file is renamed by MSWord, so you have to guess a little bit...
tpl.replace_zipname(
    "word/embeddings/Feuille_Microsoft_Office_Excel3.xlsx", "templates/real_Excel.xlsx"
)
tpl.replace_zipname(
    "word/embeddings/Pr_sentation_Microsoft_Office_PowerPoint4.pptx",
    "templates/real_PowerPoint.pptx",
)

tpl.render(context)
tpl.save("output/embedded.docx")



# -*- coding: utf-8 -*-
"""
Created : 2022-08-03
@author: Dongfang Song
"""


from docxtpl import DocxTemplate, RichText

tpl = DocxTemplate("templates/richtext_eastAsia_tpl.docx")
rt = RichText("测试TEST", font="eastAsia:Microsoft YaHei")
ch = RichText("测试TEST", font="eastAsia:微软雅黑")
sun = RichText("测试TEST", font="eastAsia:SimSun")
context = {
    "example": rt,
    "Chinese": ch,
    "simsun": sun,
}

tpl.render(context)
tpl.save("output/richtext_eastAsia.docx")



# -*- coding: utf-8 -*-
"""
Created : 2016-03-26

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate

tpl = DocxTemplate("templates/nested_for_tpl.docx")

context = {
    "dishes": [
        {"name": "Pizza", "ingredients": ["bread", "tomato", "ham", "cheese"]},
        {
            "name": "Hamburger",
            "ingredients": ["bread", "chopped steak", "cheese", "sauce"],
        },
        {
            "name": "Apple pie",
            "ingredients": ["flour", "apples", "suggar", "quince jelly"],
        },
    ],
    "authors": [
        {
            "name": "Saint-Exupery",
            "books": [
                {"title": "Le petit prince"},
                {"title": "L'aviateur"},
                {"title": "Vol de nuit"},
            ],
        },
        {
            "name": "Barjavel",
            "books": [
                {"title": "Ravage"},
                {"title": "La nuit des temps"},
                {"title": "Le grand secret"},
            ],
        },
    ],
}

tpl.render(context)
tpl.save("output/nested_for.docx")



# -*- coding: utf-8 -*-
"""
Created : 2015-03-12

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate

tpl = DocxTemplate("templates/header_footer_entities_tpl.docx")

context = {
    "title": "Header and footer test",
}

tpl.render(context)
tpl.save("output/header_footer_entities.docx")



from setuptools import setup
import os
import re
import sys

# To register onto Pypi :
# python setup.py sdist bdist_wheel upload


def read(*names):
    values = dict()
    for name in names:
        filename = name + ".rst"
        if os.path.isfile(filename):
            fd = open(filename)
            value = fd.read()
            fd.close()
        else:
            value = ""
        values[name] = value
    return values


long_description = """
%(README)s

News
====
%(CHANGES)s
""" % read(
    "README", "CHANGES"
)


def get_version(pkg):
    path = os.path.join(os.path.dirname(__file__), pkg, "__init__.py")
    if sys.version_info >= (3, 0):
        fh = open(path, encoding="utf-8")  # required to read utf-8 file on windows
    else:
        fh = open(path)  # encoding parameter does not exist in python 2
    with fh:
        m = re.search(r'^__version__\s*=\s*[\'"]([^\'"]+)[\'"]', fh.read(), re.M)
    if m:
        return m.group(1)
    raise RuntimeError("Unable to find __version__ string in %s." % path)


setup(
    name="docxtpl",
    version=get_version("docxtpl"),
    description="Python docx template engine",
    long_description=long_description,
    classifiers=[
        "Intended Audience :: Developers",
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
    keywords="jinja2",
    url="https://github.com/elapouya/python-docx-template",
    author="Eric Lapouyade",
    license="LGPL 2.1",
    packages=["docxtpl"],
    install_requires=["python-docx>=1.1.1", "docxcompose", "jinja2", "lxml"],
    extras_require={"docs": ["Sphinx", "sphinxcontrib-napoleon"]},
    eager_resources=["docs"],
    zip_safe=False,
)



import os

TEMPLATE_PATH = "templates/module_execute_tpl.docx"
JSON_PATH = "templates/module_execute.json"
OUTPUT_FILENAME = "output/module_execute.docx"
OVERWRITE = "-o"
QUIET = "-q"


if os.path.exists(OUTPUT_FILENAME):
    os.unlink(OUTPUT_FILENAME)

os.chdir(os.path.dirname(__file__))
cmd = "python -m docxtpl %s %s %s %s %s" % (
    TEMPLATE_PATH,
    JSON_PATH,
    OUTPUT_FILENAME,
    OVERWRITE,
    QUIET,
)
print('Executing "%s" ...' % cmd)
os.system(cmd)

if os.path.exists(OUTPUT_FILENAME):
    print("    --> File %s has been generated." % OUTPUT_FILENAME)



"""
Created : 2025-02-28

@author: Hannah Imrie
"""

from docxtpl import DocxTemplate, RichText, RichTextParagraph

tpl = DocxTemplate("templates/richtext_paragraph_tpl.docx")

rtp = RichTextParagraph()
rt = RichText()

rtp.add(
    "The rich text paragraph function allows paragraph styles to be added to text",
    parastyle="myrichparastyle",
)
rtp.add("Any built in paragraph style can be used", parastyle="IntenseQuote")
rtp.add(
    "or you can add your own, unlocking all style options", parastyle="createdStyle"
)
rtp.add(
    "To use, just create a style in your template word doc with the formatting you want "
    "and call it in the code.",
    parastyle="normal",
)

rtp.add("This allows for the use of")
rtp.add("custom bullet\apoints", parastyle="SquareBullet")
rtp.add("Numbered Bullet Points", parastyle="BasicNumbered")
rtp.add("and Alpha Bullet Points.", parastyle="alphaBracketNumbering")
rtp.add("You can", parastyle="normal")
rtp.add("set the", parastyle="centerAlign")
rtp.add("text alignment", parastyle="rightAlign")
rtp.add(
    "as well as the spacing between lines of text. Like this for example, "
    "this text has very tight spacing between the lines.\aIt also has no space between "
    "paragraphs of the same style.",
    parastyle="TightLineSpacing",
)
rtp.add(
    "Unlike this one, which has extra large spacing between lines for when you want to "
    "space things out a bit or just write a little less.",
    parastyle="WideLineSpacing",
)
rtp.add(
    "You can also set the background colour of a line.", parastyle="LineShadingGreen"
)

rt.add("This works with ")
rt.add("Rich ", bold=True)
rt.add("Text ", italic=True)
rt.add("Strings", underline="single")
rt.add(" too.")

rtp.add(rt, parastyle="SquareBullet")

context = {
    "example": rtp,
}

tpl.render(context)
tpl.save("output/richtext_paragraph.docx")



# -*- coding: utf-8 -*-
"""
Created : 2015-03-12

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate, RichText

tpl = DocxTemplate("templates/cellbg_tpl.docx")

context = {
    "alerts": [
        {
            "date": "2015-03-10",
            "desc": RichText("Very critical alert", color="FF0000", bold=True),
            "type": "CRITICAL",
            "bg": "FF0000",
        },
        {
            "date": "2015-03-11",
            "desc": RichText("Just a warning"),
            "type": "WARNING",
            "bg": "FFDD00",
        },
        {
            "date": "2015-03-12",
            "desc": RichText("Information"),
            "type": "INFO",
            "bg": "8888FF",
        },
        {
            "date": "2015-03-13",
            "desc": RichText("Debug trace"),
            "type": "DEBUG",
            "bg": "FF00FF",
        },
    ],
}

tpl.render(context)
tpl.save("output/cellbg.docx")



from docxtpl import DocxTemplate

tpl = DocxTemplate("templates/dynamic_table_tpl.docx")

context = {
    "col_labels": ["fruit", "vegetable", "stone", "thing"],
    "tbl_contents": [
        {"label": "yellow", "cols": ["banana", "capsicum", "pyrite", "taxi"]},
        {"label": "red", "cols": ["apple", "tomato", "cinnabar", "doubledecker"]},
        {"label": "green", "cols": ["guava", "cucumber", "aventurine", "card"]},
    ],
}

tpl.render(context)
tpl.save("output/dynamic_table.docx")



# -*- coding: utf-8 -*-
"""
Created : 2021-12-20

@author: Eric Lapouyade
"""

from docxtpl import DocxTemplate

tpl = DocxTemplate("templates/multi_rendering_tpl.docx")

documents_data = [
    {
        "dest_file": "multi_render1.docx",
        "context": {
            "title": "Title ONE",
            "body": "This is the body for first document",
        },
    },
    {
        "dest_file": "multi_render2.docx",
        "context": {
            "title": "Title TWO",
            "body": "This is the body for second document",
        },
    },
    {
        "dest_file": "multi_render3.docx",
        "context": {
            "title": "Title THREE",
            "body": "This is the body for third document",
        },
    },
]

for document_data in documents_data:
    dest_file = document_data["dest_file"]
    context = document_data["context"]
    tpl.render(context)
    tpl.save("output/%s" % dest_file)
